﻿using System;
using System.ComponentModel;
using System.Drawing;
using System.IO;
using System.Windows.Forms;

namespace Texture_Alpha_Edge_Padder
{
    public partial class fmMain : Form
    {
        public fmMain()
        {
            InitializeComponent();
        }

        private void btnPicker_Click(object sender, EventArgs e)
        {
            if (dlgFile.ShowDialog() == DialogResult.OK &&
                !dlgFile.FileName.Equals(String.Empty) &&
                !paddingTask.IsBusy)
            {
                lbStatus.Text = "Processing\nStarting";
                openedFiles = dlgFile.FileNames;
                paddingTask.RunWorkerAsync();
                btnPicker.Enabled = false;
                rdbPixel.Enabled = false;
                rdbEntire.Enabled = false;
                rdbHalf.Enabled = false;
            }
        }

        private string result;
        private string[] openedFiles;

        private void paddingTask_DoWork(object sender, DoWorkEventArgs e)
        {
            for (int file = 0; file < openedFiles.Length; file++)
            {
                try
                {
                    Bitmap image = new Bitmap(openedFiles[file]);
                    Bitmap imageInvisible = (Bitmap)image.Clone();
                    Bitmap imageInvisible2 = (Bitmap)image.Clone();
                    Graphics grD = Graphics.FromImage(imageInvisible2);
                    int w = image.Width;
                    int h = image.Height;
                    Rectangle recOld = new Rectangle(0, 0, w, h);
                    Color xpixel;
                    int max;
                    if (rdbEntire.Checked)
                    {
                        max = (int)(Math.Sqrt(Math.Max(w, h)));
                    }
                    else if (rdbHalf.Checked)
                    {
                        max = (int)(Math.Sqrt(Math.Max(w, h)) / 2);
                    }
                    else
                    {
                        max = 1;
                    }
                    for (int ii = 0; ii <= max; ii++)
                    {
                        Rectangle recUL = new Rectangle(ii, ii, w, h);
                        Rectangle recU = new Rectangle(0, ii, w, h);
                        Rectangle recUR = new Rectangle(-ii, ii, w, h);
                        Rectangle recR = new Rectangle(-ii, 0, w, h);
                        Rectangle recDR = new Rectangle(-ii, -ii, w, h);
                        Rectangle recD = new Rectangle(0, -ii, w, h);
                        Rectangle recDL = new Rectangle(ii, -ii, w, h);
                        Rectangle recL = new Rectangle(ii, 0, w, h);
                        grD.DrawImage(imageInvisible2, recUL, recOld, GraphicsUnit.Pixel);
                        grD.DrawImage(imageInvisible2, recU, recOld, GraphicsUnit.Pixel);
                        grD.DrawImage(imageInvisible2, recUR, recOld, GraphicsUnit.Pixel);
                        grD.DrawImage(imageInvisible2, recR, recOld, GraphicsUnit.Pixel);
                        grD.DrawImage(imageInvisible2, recDR, recOld, GraphicsUnit.Pixel);
                        grD.DrawImage(imageInvisible2, recD, recOld, GraphicsUnit.Pixel);
                        grD.DrawImage(imageInvisible2, recDL, recOld, GraphicsUnit.Pixel);
                        grD.DrawImage(imageInvisible2, recL, recOld, GraphicsUnit.Pixel);
                        for (int x = 0; x < w; x++)
                        {
                            for (int y = 0; y < h; y++)
                            {
                                xpixel = imageInvisible2.GetPixel(x, y);
                                if (imageInvisible.GetPixel(x, y).A == 0 && xpixel.A != 0)
                                {
                                    imageInvisible.SetPixel(x, y, Color.FromArgb(xpixel.A, xpixel.R, xpixel.G, xpixel.B));
                                }
                            }
                        }
                        result = "Progress\nFile: " + (file + 1) + "/" + openedFiles.Length + "\nWorking: " + Math.Floor((float)ii / (float)max * 50f) + "%";
                        paddingTask.ReportProgress(50);
                    }
                    Color pixel;
                    for (int x = 0; x < w; x++)
                    {
                        for (int y = 0; y < h; y++)
                        {
                            pixel = imageInvisible.GetPixel(x, y);
                            if (pixel.A != 0 && image.GetPixel(x, y).A == 0)
                            {
                                imageInvisible.SetPixel(x, y, Color.FromArgb(0, pixel.R, pixel.G, pixel.B));
                            }
                        }
                        result = "Progress\nFile: " + (file + 1) + "/" + openedFiles.Length + "\nWorking: " + Math.Floor(50 + (float)x / (float)w * 50f) + "%";
                        paddingTask.ReportProgress(50);
                    }
                    result = "Progress\nFile: " + (file + 1) + "/" + openedFiles.Length + "\nSaving File";
                    paddingTask.ReportProgress(75);
                    if (openedFiles[file].EndsWith(".png"))
                    {
                        imageInvisible.Save(openedFiles[file].Substring(0, openedFiles[file].Length-4) + "-padded.png", System.Drawing.Imaging.ImageFormat.Png);
                    }
                    else
                    {
                        imageInvisible.Save(openedFiles[file] + "-padded.png", System.Drawing.Imaging.ImageFormat.Png);
                    }
                    imageInvisible.Dispose();
                    grD.Dispose();
                    image.Dispose();
                }
                catch (Exception)
                {
                    result = "Failed!";
                    break;
                }
            }
            if (!result.Equals("Failed!"))
            {
                result = "Finished!";
            }
        }

        private void paddingTask_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            lbStatus.Text = result;
            btnPicker.Enabled = true;
            rdbPixel.Enabled = true;
            rdbEntire.Enabled = true;
            rdbHalf.Enabled = true;
        }

        private void paddingTask_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            lbStatus.Text = result;
        }

        private void btnAbout_Click(object sender, EventArgs e)
        {
            (new fmAbout()).ShowDialog();
        }

        private void fmMain_DragDrop(object sender, DragEventArgs e)
        {
            if (!paddingTask.IsBusy)
            {
                lbStatus.Text = "Processing\nStarting";
                openedFiles = (string[])e.Data.GetData(DataFormats.FileDrop);
                paddingTask.RunWorkerAsync();
                btnPicker.Enabled = false;
            }
        }

        private void fmMain_DragEnter(object sender, DragEventArgs e)
        {
            string[] files = (string[])e.Data.GetData(DataFormats.FileDrop);
            bool no_go = false;
            for (int file = 0; file < files.Length; file ++)
            {
                if (!files[file].EndsWith(".png")
                    || (File.GetAttributes(files[file]) & FileAttributes.Directory) == FileAttributes.Directory)
                {
                    no_go = true;
                    break;
                }
            }
            if (!no_go && !paddingTask.IsBusy)
            {
                if (e.Data.GetDataPresent(DataFormats.FileDrop)) e.Effect = DragDropEffects.Copy;
            }
        }
    }
}
